//Hunter & Team Hunter
//Game type imported from Tribes 2
//Code by dun_Starscaper
//Last modified 08/13/2005

$stealthDetect = true;    //Set to false to turn off M.I.B. stealth detection

//Settings the admin can change -- only set these if undefined
if($GreedEnabled == "")
  $GreedEnabled = false;  //Whether game run under greed mode
if($HoardEnabled == "")
  $HoardEnabled = false;  //Whether game run under hoard mode

//$GreedEnabled and $HoardEnabled are for map-specific settings
//To override these and establish a server-wide rule, set
//$server::GreedMode and $server::HoardMode.

//Requirements for map design
//There must be a group located in MissionGroup called "Templates".
//    It must contain a 2x2x5 trigger called "Contact" with script class "flag"
//    It must contain a flag of each color called "[Color]Flag"
//    It must contain a nav point called "navYard" with label "King's Area"
//    It must contain a nav point called "navHoard" with label "Convoy"
//There must be a collisionless visual effect (such as the teleport filler) called "Nexus/NexusBeam" in MissionGroup
//There must be a small trigger with script class "nexus" centered on MissionGroup/Nexus/NexusBeam.
//There must be a nav point called "Nexus/navNexus" with label "Nav Alpha" centered on MissionGroup/Nexus/NexusBeam.
//There must be a 400x400x400 SPHERICAL trigger with script class "camper" and contact rate 1 centered on MissionGroup/Nexus/NexusBeam


exec("trigonometryStdLib.cs");

//Globals that control the game rules
$HoardAmt   =   8;       //Number of flags a player must be carrying to be tagged as a hoarder.
$YardSize   =  12;       //Number of flags that must be dropped to call it a Yard Sale.
$YardTime   = 150;       //Number of seconds that a yard sale is marked
$GreedAmt   =   5;       //Minimum number of flags needed for capture in Greed Mode
$HoardStart = 600;       //Number of seconds before end-of-game when hoard mode starts
$HoardEnd   = 240;       //Number of seconds before end-of-game when hoard mode ends
$KillPts    =   1;       //Number of points for killing a player

$CampWarn   =   40;      //Number of seconds of staying near the nexus that the player gets warnings
$CampFreq   =    3;      //Warning frequency in seconds
$CampKill   =   60;      //Number of seconds after which player takes damage
$CampRate   =  500;      //Punishment damage rate per second

//Sounds
$YardSaleSound   = "ee_pd38.wav";			//"Luggage detected!"
$HoardStartSound = "sfx_fog.wav";
$HoardEndSound   = "sfx_fog.wav";
$GrabFlagSound   = "bigflyby.wav";
$FlagPassSound   = "sfx_shieldcap.wav";
$ScoreSound      = "sfx_electrical_bzzt.wav";
$MaxScoreSound   = "sfx_reactor_xplo.wav";

////////////////////////////////////////Setup Functions/////////////////////////

function player::onAdd(%who)
{
  if(($stealthDetect == true)&&(strAlign(1, left, getname(%player))=="/"))
  {
    say(0,0,"<f4>"@getname(%player)@" joined the game","alarm2.wav");
    messagebox(%player,"This server is equipped with an anti-stealth detection system. The slash on your name has "
      @ "been detected. All players have been alerted to your presence.");
  }

  if(PlayerManager::getPlayerCount() == 1)
    setRules();

  DataStore(%who, "Score", 0);
  DataStore(%who, "Caps",  0);
  
  if($server::TeamPlay)
    say(%who, 0, "<F1>Welcome to Team Hunter! Check the Game Info tab for rules and scoring information.");
  else
    say(%who, 0, "<F1>Welcome to Hunter! Check the Game Info tab for rules and scoring information.");
  say(%who, 0, "Hunter and Team Hunter developed by Sierra for Tribes 2. Game design ported to Starsiege by " @
               "dun_Starscaper (dun_Starscaper@yahoo.com).");
  
  player::onAddLog(%who);
}

function player::onRemove(%who)
{
  if(($stealthDetect == true)&&(strAlign(1, left, getname(%player))=="/"))
    say(0,0,"<f4>"@getname(%player)@" left the game","alarm2.wav");

  player::onRemoveLog(%who);
}

function vehicle::onAdd(%what)
{
  DataStore(%what, "Flags", 0);
  DataStore(%what, "Hoard", 0);
  DataStore(%what, "Camp",  0);
  
  if(isTimeLimit())
  {
    %totalSeconds = $server::TimeLimit * 60;
    %player = PlayerManager::vehicleIDToPlayerNum(%what);
    
    %left = %totalSeconds - (getCurrentTime() - $MissionStartTime);
    setHUDTimer(%left, -1, "Match Ends", 1, %player);
    
    if(isHoardModeEnabled() && (%left <= $HoardStart) && (%left >= $HoardEnd))
      setHUDTimer(%left - $HoardEnd, -1, "Hoard Period Ends", 2, %player);
  }
  
  setNavMarker("MissionGroup\\Nexus\\navNexus", true, %what);
}

function onMissionStart()
{
  setNexus(true);
  $MissionStartTime = getCurrentTime();
  $MaxScore         = 0;
  if($server::TeamPlay)
      $MaxTeamScore = 0;
  
  for(%i = 0; %i < PlayerManager::getPlayerCount(); %i++)
  {
    %player = PlayerManager::getPlayerNum(%i);
    DataStore(%player, "Score", 0);
    DataStore(%player, "Caps",  0);
  }
  
  if($server::TeamPlay)
  {
    $TeamScore[*IDSTR_TEAM_YELLOW] = 0;
    $TeamScore[*IDSTR_TEAM_BLUE]   = 0;
    $TeamScore[*IDSTR_TEAM_PURPLE] = 0;
    $TeamScore[*IDSTR_TEAM_RED]    = 0;
    $TeamCaps[*IDSTR_TEAM_YELLOW]  = 0;
    $TeamCaps[*IDSTR_TEAM_BLUE]    = 0;
    $TeamCaps[*IDSTR_TEAM_PURPLE]  = 0;
    $TeamCaps[*IDSTR_TEAM_RED]     = 0;
  }
  
  if(isTimeLimit())
  {
    %totalSeconds = $server::TimeLimit * 60;
    schedule("preGameEnd();", %totalSeconds - 60);
    
    if(isHoardModeEnabled())
    {
      %timeToHoard = %totalSeconds - $HoardStart;
      %timeToDump  = %totalSeconds - $HoardEnd;
      schedule("preHoardStart();", %timeToHoard - 30);
      schedule("preHoardEnd();",   %timeToDump - 30);
    }
  }
  
  logStandardMissionStart();
  if(isHoardModeEnabled()) missionMsg("HOARD mode is ENABLED"); else missionMsg("HOARD mode is DISABLED");
  if(isGreedModeEnabled()) missionMsg("GREED mode is ENABLED"); else missionMsg("GREED mode is DISABLED");
}

function setRules()
{
  %rules = "<F2>GameType\n<F0>";
  if($server::TeamPlay)
    %rules = %rules @ "Team ";
  %rules = %rules @ "Hunter\n\n"
                  @ "<F2>Mission\n<F0>" @ $missionName @ "\n\n"
                  @ "<F2>Synopsis\n<F0>Collect flags from your enemies and return them for points.\n\n"
                  @ "<F2>Rules\n<F0>Players drop flags (including any they were carrying) when they die. Another "
                  @ "player can collect the flags by running over them.\n\n"
                  @ "Players can redeem flags for points by running through the <F3>Nexus<F0> at Nav Alpha";
  if($server::TeamPlay)
    %rules = %rules @ ", or pass flags to their teammates by scanning them";
  %rules = %rules @ ".\n\n";
  if(isGreedModeEnabled())
    %rules = %rules @ "<F3>Greed<F0> mode is enabled. Players cannot score until they have at least <F3>" @ $GreedAmt
                    @ "<F0> flags.\n\n";
  if((isHoardModeEnabled())&&(isTimeLimit()))
    %rules = %rules @ "<F3>Hoard<F0> mode is enabled. There will be a period <F3>" @ timeDifference($HoardStart, $HoardEnd)
                    @ "<F0> long towards the end of the match when nobody can score.\n\n";
  %rules = %rules @ "<F2>Scoring\n<F0>Players get <F3>" @ $KillPts @ "<F0> point for killing another player.\n\n"
                  @ "When flags are returned to the Nexus, their value increases with the number of flags the player is carrying. "
                  @ "For example, 2 flags are worth <F3>" @ flagsToPoints(2) @ "<F0> points, but 4 flags are worth <F3>"
                  @ flagsToPoints(4) @ "<F0> points.\n\n";
//  if($server::TeamPlay)
//    %rules = %rules @ "The team's score is the sum of its members' scores.\n\n";
  
  %rules = %rules @ "<tIDMULT_STD_ITEMS>"    @
                    "<bctf1.bmp><F3>Flags:<F0> Collect flags by running over them.\n\n\n\n\n\n\n" @
                    "<btele1.bmp><F3>Nexus:<F0> Run through it while carrying flags to score points.\n\n\n\n\n\n" @
                    "<tIDMULT_STD_ZEN_1>"    @ $ZenWaitTime @ "<tIDMULT_STD_ZEN_2>";

   setGameInfo(%rules);
}

////////////////////////////////////////Basic Actions///////////////////////////

function getFlags(%veh, %amt)
{
  %numFlags = DataRetrieve(%veh, "Flags") + %amt;
  DataStore(%veh, "Flags", %numFlags);
  
  if(%numFlags >= $HoardAmt)
  {
    %nav = DataRetrieve(%veh, "Hoard");
    if(%nav == 0)
    {
      %nav = makeHoardBeacon(getTeam(%veh));
      DataStore(%veh, "Hoard", %nav);
      setVehicleSpecialIdentity(%veh, true);
    }
    warp(%nav, %veh);
  }
  notifyFlagCount(%veh);
}

function loseFlags(%veh, %amt)
{
  %numFlags = DataRetrieve(%veh, "Flags") - %amt;
  if(%numFlags < 0) %numFlags = 0;
  DataStore(%veh, "Flags", %numFlags);
  
  if(%numFlags < $HoardAmt)
  {
    %nav = DataRetrieve(%veh, "Hoard");
    if(%nav != 0)
    {
      deleteObject(%nav);
      DataStore(%veh, "Hoard", 0);
      setVehicleSpecialIdentity(%veh, false);
    }
  }
}

function makeFlag(%team, %x, %y)
{
  %z = getTerrainHeight(%x, %y);
  
  %color   = teamToColor(%team);
  %flag    = copyObject(getObjectID("MissionGroup\\Templates\\" @ %color @ "Flag"));
  %trigger = copyObject(getObjectID("MissionGroup\\Templates\\Contact"));
  
  setPosition(%flag, %x, %y, %z);
  setPosition(%trigger, %x, %y, %z);

  schedule("playAnimSequence(" @ %flag @ ", 0, 0);", 1);    //Delay anim, or won't be visible
  DataStore(%trigger, "Marker", %flag);
}

function setNexus(%status)
{
  setShapeVisibility(getObjectID("MissionGroup\\Nexus\\NexusBeam"), %status);
  $NexusOnline = %status;
  missionLog("Nexus status set to ", %status);
}

function dumpFlags(%vehicle)
{
  %flags = DataRetrieve(%vehicle, "Flags");
  %team  = getTeam(%vehicle);
  loseFlags(%vehicle, %flags);
  
  missionLog(getHudName(%vehicle), " dropped ", %flags + 1, " flags.");

  for(%i = 0; %i < %flags; %i++)
  {
    %angle = randomInt(0, 359);
    %dist  = randomDist(50);
    
    %x = getPosition(%vehicle, x) + %dist*cos(%angle);
    %y = getPosition(%vehicle, y) + %dist*sin(%angle);
    makeFlag(%team, %x, %y);
  }
  
  if(%flags >= $YardSize - 1)
  {
    %nav = makeYardBeacon();
    warp(%nav, %vehicle);
    say(0, getSoundChannel(), "YARD SALE!!!!", $YardSaleSound);
    missionMsg("Yard sale!");
    
    schedule("deleteObject(" @ %nav @ ");", $YardTime);
  }
}

function pardon(%vehicle)
{
  //Don't pardon if player re-entered the region -- will resume when player leaves
  if(DataRetrieve(%vehicle, "Camping") == true) return;
  
  %numCamps = DataRetrieve(%vehicle, "Camp") - 1;
  DataStore(%vehicle, "Camp", %numCamps);
  
  if(%numCamps > 0) schedule("pardon(" @ %vehicle @ ");", 2);
}

////////////////////////////////////////Gameplay Functions//////////////////////

function flag::trigger::onEnter(%where, %who)
{
  %flag = DataRetrieve(%where, "Marker");
  setPosition(%where, getPosition(%where, x), getPosition(%where, y), getPosition(%where, z) - 1000);
  schedule("DeleteObject(" @ %where @ ");", 1);
  DeleteObject(%flag);
  getFlags(%who, 1);
  say(PlayerManager::vehicleIDToPlayerNum(%who), getSoundChannel(), $GrabFlagSound);    //Random channel allows superposition
}

function vehicle::onDestroyed(%destroyee, %destroyer)
{
  makeFlag(getTeam(%destroyee), getPosition(%destroyee, x), getPosition(%destroyee, y));
  dumpFlags(%destroyee);
  
  if(%destroyee != %destroyer)
  {
    if($server::TeamPlay && (getTeam(%destroyer) == getTeam(%destroyee)))
    {
      damageObject(%destroyee, 30000);
      say(0, 0, "<F5>" @ getHUDName(%destroyer) @ " turned traitor on " @ getHUDName(%destroyee) @ ".");
    }
    else
    {
      incrementScore(PlayerManager::VehicleIDToPlayerNum(%destroyer), $KillPts);
      %msg = GetFancyDeathMessage(getHUDName(%destroyee), getHUDName(%destroyer));
      if(%msg != "") say(0, 0, %msg);
    }
  }
  else
    say(0, 0, getHUDName(%destroyee) @ " died.");
  
  vehicle::onDestroyedLog(%destroyee, %destroyer);
}

function nexus::trigger::onEnter(%where, %who)
{
  %player = PlayerManager::vehicleIDToPlayerNum(%who);
  %flags  = dataRetrieve(%who, "Flags");
  if(%flags == 0)
    say(%player, 0, "<F4>Get flags by killing players and return them here to score.");
  else if(!$NexusOnline)
    say(%player, 0, "<F4>The nexus has been disabled. You may not score.");
  else if(isGreedModeEnabled() && %flags < $GreedAmt)
    say(%player, 0, "<F4>Greed mode is enabled. You need " @ $GreedAmt @" flags to score but only have " @ %flags @ ".");
  else
  {
    %points = flagsToPoints(%flags);
    incrementScore(%player, %points);
    incrementCaps(%player, %flags);
    loseFlags(%who, %flags);
    
    if((%flags == 1)&&(%points == 1))
      say(0, 0, getName(%player) @ " has captured 1 flag for 1 point!");
    else if(%points == 1)
      say(0, 0, getName(%player) @ " has captured " @ %flags @ " flags for 1 point!");
    else if(%flags == 1)
      say(0, 0, getName(%player) @ " has captured 1 flag for " @ %points @ " points!");
    else
      say(0, 0, getName(%player) @ " has captured " @ %flags @ " flags for " @ %points @ " points!");
      
    missionLog(getName(%player) @ " captured " @ %flags @ " flags for " @ %points @ " points.");

    if($server::TeamPlay)
    {
      %totalScore = $TeamScore[getTeam(%player)];
      if(%totalScore > $MaxTeamScore)
      {
        $MaxTeamScore = %totalScore;
        say(0, 0, getTeam(%player) @ " is now leading with a score of " @ $MaxTeamScore @ ".", $MaxScoreSound);
      }
      else
        say(%player, 0, $ScoreSound);
    }
    else
    {
      %totalScore = getPlayerScore(%player);
      if(%totalScore > $MaxScore)
      {
        $MaxScore = %totalScore;
        say(0, 0, getName(%player) @ " is now leading with a score of " @ $MaxScore @ ".", $MaxScoreSound);
      }
      else
        say(%player, 0, $ScoreSound);
    }
  }
}

function vehicle::onScan(%target, %scanner)
{
  if(!$server::TeamPlay) return;
  
  %passer  = PlayerManager::vehicleIDToPlayerNum(%scanner);
  %receive = PlayerManager::vehicleIDToPlayerNum(%target);
  if(getTeam(%target) != getTeam(%scanner))
    say(%passer, 0, "<F4>You can't pass flags to the other team!");
  else
  {
    %flags = dataRetrieve(%scanner, "Flags");
    getFlags(%target, %flags);
    loseFlags(%scanner, %flags);
    say(0, 0, getHUDName(%scanner) @ " passed " @ %flags @ " flags to " @ getHUDName(%target) @ ".");
    say(%passer,  0, $FlagPassSound);
    say(%receive, 0, $FlagPassSound);
  }
}

function camper::trigger::onEnter(%where, %who)
{
  DataStore(%who, "Camping", true);
}

function camper::trigger::onContact(%where, %who)
{
  %player   = PlayerManager::VehicleIDToPlayerNum(%who);
  %numCamps = DataRetrieve(%who, "Camp") + 1;
  DataStore(%who, "Camp", %numCamps);
  
  if((%numCamps > $CampWarn)&&(%numCamps % $CampFreq == 0))
    if(%player != 0) say(%player, 0, "No camping near the nexus.");
  if(%numCamps > $CampKill)
  {
    damageObject(%who, $CampRate);
    if(%player != 0)
    {
      fadeEvent(          %player, out, 0.2, 1, 0, 0);
      schedule("fadeEvent(" @ %player @ ",  in, 0.2, 1, 0, 0);", 0.2);
    }
  }
}

function camper::trigger::onLeave(%where, %who)
{
  DataStore(%who, "Camping", false);
  schedule("pardon(" @ %who @ ");", 5);
}

function preHoardStart()
{
  schedule("say(0, 0, \"<F4>HOARD mode begins in 30 seconds.\");",  0);
  schedule("say(0, 0, \"<F4>HOARD mode begins in 10 seconds!\");", 20);
  for(%i = 5; %i >= 1; %i--)
    schedule("say(0, 0, \"<F4>" @ %i @ "...\");", 30 - %i);
  schedule("onHoardStart();", 30);
}

function onHoardStart()
{
  setNexus(false);
  say(0, getSoundChannel(), "<F4>HOARD mode has begun. Players can no longer score.", $HoardStartSound);
  missionLog("Hoard mode began.");
  
  for(%i = 0; %i < PlayerManager::getPlayerCount(); %i++)
  {
    %player = PlayerManager::getPlayerNum(%i);
    if(%player != 0)
    {
      %vehicle = PlayerManager::playerNumToVehicleID(%player);
      if(%vehicle != 0)
        setHUDTimer($HoardStart - $HoardEnd, -1, "Hoard Period Ends", 2, %player);
    }
  }
}

function preHoardEnd()
{
  schedule("say(0, 0, \"<F4>HOARD mode ends in 30 seconds.\");",  0);
  schedule("say(0, 0, \"<F4>HOARD mode ends in 10 seconds!\");", 20);
  for(%i = 5; %i >= 1; %i--)
    schedule("say(0, 0, \"<F4>" @ %i @ "...\");", 30 - %i);
  schedule("onHoardEnd();", 30);
}

function onHoardEnd()
{
  setNexus(true);
  say(0, getSoundChannel(), "<F4>HOARD mode has ended.", $HoardEndSound);
  missionLog("Hoard mode ended.");
}

function preGameEnd()
{
  schedule("say(0, 0,  \"1min_2mis_end.wav\");",  0);
  schedule("say(0, 0, \"30sec_2mis_end.wav\");", 30);
  schedule("say(0, 0, \"Match ends in 10 seconds!\");", 50);
  for(%i = 5; %i >= 1; %i--)
    schedule("say(0, 0, \"" @ %i @ "...\");", 60 - %i);
}

function onMissionEnd()
{
  logStandardMissionEnd();
  
  //Gotta get ridda those flags!
  for(%i = getNextObject("MissionCleanup", 0); %i != 0; %i = getNextObject("MissionCleanup", 0))
    deleteObject(%i);
}

////////////////////////////////////////Scoreboard Functions////////////////////

function getPlayerScore(%player)
{
  return DataRetrieve(%player, "Score");
}

function getFlagsCapped(%player)
{
  return DataRetrieve(%player, "Caps");
}

function getFlagsCarried(%player)
{
  %vehicle = PlayerManager::playerNumToVehicleID(%player);
  if(%vehicle == 0)
    return "";
  else
    return DataRetrieve(%vehicle, "Flags");
}

function getTeamFlagsCapped(%bits)
{
  %score = $TeamCaps[getTeamNameFromTeamId(%bits)];
  if(%score != "")
    return %score;
  else
    return 0;
}

function getTeamScore(%bits)
{
  %score = $TeamScore[getTeamNameFromTeamId(%bits)];
  if(%score != "")
    return %score;
  else
    return 0;
}

function initScoreBoard()
{
   deleteVariables("$ScoreBoard::PlayerColumn*");
   deleteVariables("$ScoreBoard::TeamColumn*");

   if($server::TeamPlay == "True")	
   {
	   // Player ScoreBoard column headings
	   $ScoreBoard::PlayerColumnHeader1 = *IDMULT_SCORE_TEAM;
	   $ScoreBoard::PlayerColumnHeader2 = *IDMULT_SCORE_SQUAD;
	   $ScoreBoard::PlayerColumnHeader3 = *IDMULT_SCORE_SCORE;
	   $ScoreBoard::PlayerColumnHeader4 = "Flags";
	   $ScoreBoard::PlayerColumnHeader5 = *IDMULT_SCORE_KILLS;
	   $ScoreBoard::PlayerColumnHeader6 = *IDMULT_SCORE_DEATHS;

	   // Player ScoreBoard column functions
	   $ScoreBoard::PlayerColumnFunction1 = "getTeam";
	   $ScoreBoard::PlayerColumnFunction2 = "getSquad";
	   $ScoreBoard::PlayerColumnFunction3 = "getPlayerScore";
	   $ScoreBoard::PlayerColumnFunction4 = "getFlagsCarried";
	   $ScoreBoard::PlayerColumnFunction5 = "getKills";
	   $ScoreBoard::PlayerColumnFunction6 = "getDeaths";

     // Team ScoreBoard column headings
     $ScoreBoard::TeamColumnHeader1 = *IDMULT_SCORE_SCORE;
     $ScoreBoard::TeamColumnHeader2 = *IDMULT_SCORE_PLAYERS;
     $ScoreBoard::TeamColumnHeader3 = *IDMULT_SCORE_KILLS;
     $ScoreBoard::TeamColumnHeader4 = *IDMULT_SCORE_DEATHS;

     // Team ScoreBoard column functions
     $ScoreBoard::TeamColumnFunction1 = "getTeamScore";
     $ScoreBoard::TeamColumnFunction2 = "getNumberOfPlayersOnTeam";
     $ScoreBoard::TeamColumnFunction3 = "getTeamKills";
     $ScoreBoard::TeamColumnFunction4 = "getTeamDeaths";
   }
   else
   {
     // Player ScoreBoard column headings
	   $ScoreBoard::PlayerColumnHeader1 = *IDMULT_SCORE_SQUAD;
	   $ScoreBoard::PlayerColumnHeader2 = *IDMULT_SCORE_SCORE;
	   $ScoreBoard::PlayerColumnHeader3 = "Flags";
	   $ScoreBoard::PlayerColumnHeader4 = *IDMULT_SCORE_KILLS;
	   $ScoreBoard::PlayerColumnHeader5 = *IDMULT_SCORE_DEATHS;

	   // Player ScoreBoard column functions
	   $ScoreBoard::PlayerColumnFunction1 = "getSquad";
	   $ScoreBoard::PlayerColumnFunction2 = "getPlayerScore";
	   $ScoreBoard::PlayerColumnFunction3 = "getFlagsCarried";
	   $ScoreBoard::PlayerColumnFunction4 = "getKills";
	   $ScoreBoard::PlayerColumnFunction5 = "getDeaths";
   }

   // tell server to process all the scoreboard definitions defined above
   serverInitScoreBoard();
}


////////////////////////////////////////Utility Functions///////////////////////

function copyObject(%object, %name)
{
   storeObject(%object, "copyObject.tmp");
   if(%name == "")
   {
      %copy = loadObject(getObjectName(%object), "copyObject.tmp");
   }
   else
   {
      %copy = loadObject(%name, "copyObject.tmp");
   }
   addToSet("MissionCleanup", %copy);
   return %copy;
}

function teamToColor(%team)
{
  if(%team == *IDSTR_TEAM_YELLOW) return "Yellow";
  if(%team == *IDSTR_TEAM_RED) return "Red";
  if(%team == *IDSTR_TEAM_PURPLE) return "Purple";
  if(%team == *IDSTR_TEAM_BLUE) return "Blue";
  
  return "";
}

function makeHoardBeacon(%team)
{
  %nav = copyObject("MissionGroup\\Templates\\navHoard");
  
  if($server::TeamPlay)
    setTeam(%nav, %team);
  setNavMarker(%nav, true, 0);
  return %nav;
}

function makeYardBeacon()
{
  %nav = copyObject("MissionGroup\\Templates\\navYard");
  
  setNavMarker(%nav, true, -1);
  return %nav;
}

function notifyFlagCount(%who)
{
  dataStore(%who, "LastFlag", getSimTime());
  schedule("announceFlagCount(" @ %who @ ");", 5);
}

//Need to use decimal time to resolve closely spaced flag grabs
function announceFlagCount(%who)
{
  %now  = getSimTime();
  %then = DataRetrieve(%who, "LastFlag");
  if(%now - %then < 4.999)
    return;       //A later call was made
  else
  {
    DataRelease(%who, "LastFlag");
    %player = PlayerManager::vehicleIDToPlayerNum(%who);
    if(%player != 0)
    {
      if(DataRetrieve(%who, "Flags") == 1)
        say(%player, 0, "<F4>You now have 1 flag.");
      else
        say(%player, 0, "<F4>You now have " @ DataRetrieve(%who, "Flags") @ " flags.");
    }
  }
}

function incrementScore(%player, %amount)
{
  DataStore(%player, "Score", DataRetrieve(%player, "Score") + %amount);
  if($server::TeamPlay)
    $TeamScore[getTeam(%player)]+= %amount;
}

function incrementCaps(%player, %amount)
{
  DataStore(%player, "Caps", DataRetrieve(%player, "Caps") + %amount);
  if($server::TeamPlay)
    $TeamCaps[getTeam(%player)]+= %amount;
}

function flagsToPoints(%flags)
{
  return %flags * (%flags + 1) / 2;
}

function isTimeLimit()
{
  return (($server::TimeLimit != 0)&&($server::TimeLimit != 1440));
}

function isHoardModeEnabled()
{
  if($server::HoardMode == "")
    return $HoardEnabled;
  else
    return $server::HoardMode;
}

function isGreedModeEnabled()
{
  if($server::GreedMode == "")
    return $GreedEnabled;
  else
    return $server::GreedMode;
}

function getSoundChannel()
{
    return randomInt(200, 300);
}

function randomDist(%mean)
{
  //Want a distribution that gives a large chance of being near, and some smaller chance of being far
  //Use inverse distribution function method (only one I know)
  //An exponential distribution would be nice: x =  -- Can't do logs in SSSL
  //A quadratic would be nice, too: F(x) = -(x^3)/3 + ux^2  --  Can't find inverse
  
  //Settled for linear -- f(x) = x/u^2 (0 <= x <= u), (2u-x)/u^2 (u <= x <= 2u)
  //Inverse is x = sqrt(2(u^2)F)                F < 0.5
  //             = 2u - sqrt(2(u^2)(1 - F))     F > 0.5
  
  %sum = randomFloat(0, 1);
  
  if(%sum <= 0.5)
    return %mean*sqrt(2 * %sum);
  else
    return 2*%mean - %mean*sqrt(2 * (1-%sum));
}

setRules();